package setup

import (
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/collections"
	"github.com/stretchr/testify/assert"
)

func TestEntityList_WithOptions(t *testing.T) {
	colls := make(Collections, 0)
	colls.Add(&collections.Collection{ID: "1", SpaceID: "sp", EnvID: "env"})
	colls.Add(&collections.Collection{ID: "2", SpaceID: "sp", EnvID: "env"})
	colls.Add(&collections.Collection{ID: "3", SpaceID: "sp", EnvID: "env"})
	colls.Add(&collections.Collection{ID: "4", SpaceID: "sp", EnvID: "env"})
	colls.WithOptions(func(c *collections.Collection) bool { return c.ID == "1" || c.ID == "3" }, func(c *Collection) {
		c.UpdateFunc = nil
	})

	assert.Nil(t, colls[0].UpdateFunc)
	assert.NotNil(t, colls[1].UpdateFunc)
	assert.Nil(t, colls[2].UpdateFunc)
	assert.NotNil(t, colls[3].UpdateFunc)
}

func TestEntityList_Add(t *testing.T) {
	c := &collections.Collection{ID: "1", SpaceID: "sp", EnvID: "env"}
	colls := make(Collections, 0)
	colls.Add(c)
	colls.Add(c)
	assert.Equal(t, colls[0].value, c)
	assert.Equal(t, colls[1].value, c)
	assert.True(t, colls[0].value != c)
	assert.True(t, colls[1].value != c)
	assert.True(t, colls[0].value != colls[1].value)
}

func TestEntityList_Get(t *testing.T) {
	c := &collections.Collection{ID: "1", SpaceID: "sp", EnvID: "env"}
	colls := make(Collections, 0)
	colls.Add(c)
	c1 := colls[0].Value(nil)
	assert.Equal(t, c, c1)
	assert.Equal(t, colls[0].value, c1)
	assert.True(t, c != c1)
	assert.True(t, colls[0].value != c1)
}
